@@ -25,17 +25,17 @@ from optparse import OptionParser, OptionError
 import logging
 
 import common
-from common import FDroidPopen
+from common import FDroidPopen, FDroidException
 
 options = None
 config = None
 
 
 def devices():
-    p = FDroidPopen(["adb", "devices"])
+    p = FDroidPopen([config['adb'], "devices"])
     if p.returncode != 0:
-        raise Exception("An error occured when finding devices: %s" % p.stdout)
-    lines = p.stdout.splitlines()
+        raise FDroidException("An error occured when finding devices: %s" % p.output)
+    lines = p.output.splitlines()
     if lines[0].startswith('* daemon not running'):
         lines = lines[2:]
     if len(lines) < 3:
@@ -85,7 +85,7 @@ def main():
 
         for appid, apk in apks.iteritems():
             if not apk:
-                raise Exception("No signed apk available for %s" % appid)
+                raise FDroidException("No signed apk available for %s" % appid)
 
     else:
 
@@ -96,13 +96,13 @@ def main():
         # Get device list each time to avoid device not found errors
         devs = devices()
         if not devs:
-            raise Exception("No attached devices found")
+            raise FDroidException("No attached devices found")
         logging.info("Installing %s..." % apk)
         for dev in devs:
             logging.info("Installing %s on %s..." % (apk, dev))
-            p = FDroidPopen(["adb", "-s", dev, "install", apk])
+            p = FDroidPopen([config['adb'], "-s", dev, "install", apk])
             fail = ""
-            for line in p.stdout.splitlines():
+            for line in p.output.splitlines():
                 if line.startswith("Failure"):
                     fail = line[9:-1]
             if not fail:
@@ -111,7 +111,7 @@ def main():
             if fail == "INSTALL_FAILED_ALREADY_EXISTS":
                 logging.warn("%s is already installed on %s." % (apk, dev))
             else:
-                raise Exception("Failed to install %s on %s: %s" % (
+                raise FDroidException("Failed to install %s on %s: %s" % (
                     apk, dev, fail))
 
     logging.info("\nFinished")
