@@ -25,7 +25,7 @@ import logging
 
 from collections import OrderedDict
 
-srclibs = {}
+srclibs = None
 
 
 class MetaDataException(Exception):
@@ -154,9 +154,14 @@ class FieldValidator():
 valuetypes = {
     FieldValidator("Integer",
                    r'^[1-9][0-9]*$', None,
-                   ['FlattrID'],
+                   [],
                    ['vercode']),
 
+    FieldValidator("Hexadecimal",
+                   r'^[0-9a-f]+$', None,
+                   ['FlattrID'],
+                   []),
+
     FieldValidator("HTTP link",
                    r'^http[s]?://', None,
                    ["Web Site", "Source Code", "Issue Tracker", "Donate"], []),
@@ -451,6 +456,11 @@ def read_srclibs():
     metadata.
     """
     global srclibs
+
+    # They were already loaded
+    if srclibs is not None:
+        return
+
     srclibs = {}
 
     srcdir = 'srclibs'
@@ -465,6 +475,11 @@ def read_srclibs():
 # Read all metadata. Returns a list of 'app' objects (which are dictionaries as
 # returned by the parse_metadata function.
 def read_metadata(xref=True):
+
+    # Always read the srclibs before the apps, since they can use a srlib as
+    # their source repository.
+    read_srclibs()
+
     apps = []
 
     for basedir in ('metadata', 'tmp'):
@@ -487,7 +502,7 @@ def read_metadata(xref=True):
         for app in apps:
             try:
                 description_html(app['Description'], linkres)
-            except Exception, e:
+            except MetaDataException, e:
                 raise MetaDataException("Problem with description of " + app['id'] +
                                         " - " + str(e))
 
@@ -523,6 +538,23 @@ def flagtype(name):
     return 'string'
 
 
+def fill_build_defaults(build):
+
+    def get_build_type():
+        for t in ['maven', 'gradle', 'kivy']:
+            if build[t]:
+                return t
+        if build['output']:
+            return 'raw'
+        return 'ant'
+
+    for flag, value in flag_defaults.iteritems():
+        if flag in build:
+            continue
+        build[flag] = value
+    build['type'] = get_build_type()
+
+
 # Parse metadata for a single application.
 #
 #  'metafile' - the filename to read. The package id for the application comes
@@ -618,14 +650,6 @@ def parse_metadata(metafile):
             thisinfo['comments'].append((key, comment))
         del curcomments[:]
 
-    def get_build_type(build):
-        for t in ['maven', 'gradle', 'kivy']:
-            if build[t]:
-                return t
-        if build['output']:
-            return 'raw'
-        return 'ant'
-
     thisinfo = {}
     if metafile:
         if not isinstance(metafile, file):
@@ -757,11 +781,7 @@ def parse_metadata(metafile):
         thisinfo['Description'].append('No description available')
 
     for build in thisinfo['builds']:
-        for flag, value in flag_defaults.iteritems():
-            if flag in build:
-                continue
-            build[flag] = value
-        build['type'] = get_build_type(build)
+        fill_build_defaults(build)
 
     return thisinfo
 
@@ -779,7 +799,7 @@ def write_metadata(dest, app):
                 mf.write("%s\n" % comment)
                 written += 1
         if written > 0:
-            logging.debug("...writing comments for " + (key if key else 'EOF'))
+            logging.debug("...writing comments for " + (key or 'EOF'))
 
     def writefield(field, value=None):
         writecomments(field)
@@ -831,6 +851,10 @@ def write_metadata(dest, app):
         writefield('Repo')
         mf.write('\n')
     for build in app['builds']:
+
+        if build['version'] == "Ignore":
+            continue
+
         writecomments('build:' + build['vercode'])
         mf.write("Build:%s,%s\n" % (build['version'], build['vercode']))
 
