@@ -24,8 +24,7 @@ import logging
 
 import common
 import metadata
-from common import BuildException
-from common import VCSException
+from common import BuildException, VCSException
 
 config = None
 options = None
@@ -41,8 +40,6 @@ def main():
                       help="Spew out even more information than normal")
     parser.add_option("-q", "--quiet", action="store_true", default=False,
                       help="Restrict output to warnings and errors")
-    parser.add_option("--nosvn", action="store_true", default=False,
-                      help="Skip svn repositories - for test purposes, because they are too slow.")
     (options, args) = parser.parse_args()
 
     config = common.read_config(options)
@@ -50,7 +47,6 @@ def main():
     # Read all app and srclib metadata
     allapps = metadata.read_metadata()
     apps = common.read_app_args(args, allapps, True)
-    metadata.read_srclibs()
 
     problems = []
 
@@ -69,8 +65,6 @@ def main():
         if not app['builds']:
             logging.info("Skipping %s: no builds specified" % app['id'])
             continue
-        elif options.nosvn and app['Repo Type'] == 'svn':
-            continue
 
         logging.info("Processing " + app['id'])
 
